#include <stdint.h>
#include "fip_urv_regs.h"
#include "fip_urv_cmd.h"

static unsigned extract_word (volatile uint32_t *buf)
{
  unsigned res;

  /* Little endian.  */
  res = (buf[0] & 0xff) << 0;
  res |= (buf[1] & 0xff) << 8;
  res |= (buf[2] & 0xff) << 16;
  res |= (buf[3] & 0xff) << 24;

  return res;
}

static void insert_word(unsigned char *buf, unsigned val)
{
  buf[0] = (val >> 0) & 0xff;
  buf[1] = (val >> 8) & 0xff;
  buf[2] = (val >> 16) & 0xff;
  buf[3] = (val >> 24) & 0xff;
}

int
main (void)
{
  volatile struct fip_urv_regs *regs =
    (volatile struct fip_urv_regs *)0x100000;
  int j;
  unsigned char reply[8];
  unsigned char rdy;
  unsigned int leds;

  /* Init.  */
  leds = 1;
  j = 0;
  rdy = 0;

  /* Infinite loop.  */
  while (1)
    {
      /* Check for FIP message.  */
      unsigned int status = regs->fip_status;

      /* Control leds.  */
      if (leds & 0x80)
        regs->leds = leds;
      else
        {
          regs->leds = (leds << 2) | status;
          j++;
          if (j == 1000000)
            {
              leds = (leds >> 1) | ((leds & 1) << 3);
              j = 0;
            }
        }


      /* Send leds status.  */
      if (status & 1)
        {
          /* Received.  */
          unsigned char cmd;

          /* Consume variable.  */
          regs->fip_var1 = 1;
          cmd = regs->fip_reg[0x02];

          /* Default reply: command + id.  */
          reply[0] = cmd;
          reply[1] = regs->fip_reg[0x03];

          switch (cmd)
            {
            case CMD_SET_LEDS:
              leds = regs->fip_reg[0x06];
              break;
            case CMD_READ_WORD:
              {
                unsigned *addr;

                addr = (unsigned *)extract_word(&regs->fip_reg[0x6]);
                insert_word(&reply[4], *addr);
              }
              break;
            case CMD_WRITE_BLOCK:
            case CMD_WRITE_WORD:
              {
                unsigned *addr;
                unsigned val;
                unsigned k;

                addr = (unsigned *)extract_word(&regs->fip_reg[0x6]);
                for (k = 0; k < cmd; k++)
                  {
                    val = extract_word(&regs->fip_reg[0xa + 4*k]);
                    addr[k] = val;
                  }
              }
              break;
            default:
              reply[0] = CMD_ERROR;
              break;
            }
          rdy = 1;
          regs->fip_var1 = 0;
        }

      if ((status & 2) && rdy)
        {
          int i;

          /* Produce variable.  */
          regs->fip_var3 = 1;

          for (i = 0; i < sizeof(reply); i++)
            regs->fip_reg[0x102 + i] = reply[i];

          regs->fip_var3 = 0;

          rdy = 0;
        }
    }
}
